#!/bin/bash
# Script to create the freeswan kernel patch with x509 support
# Needs to have both the freeswan and the kernel .src.rpm installed.
# Erlend Midttun, <erlendbm@trustix.com>
#
# Usage: $0 kernel-version freeswan-version x509-version
#   i.e. $0 2.4.20 2.00 1.3.2
#

# Where to find the SOURCES directory
RPMROOT=/usr/src/trustix

if [ $# -ne 3 ]; then
echo "Usage: $0 kernel-version freeswan-version x509-version"
echo "     i.e. $0 2.4.20 2.00 1.3.2"
  exit 1
fi

# Kernel version
KVERSION=$1
# FreeSWAN version
FVERSION=$2
# x509 patch version
XVERSION=$3

FREESWANPATCH=$RPMROOT/SOURCES/kernel-$1-freeswan-$2-x509-$3.patch.bz2
KERNELSOURCE=$RPMROOT/SOURCES/linux-$KVERSION.tar.bz2
FREESWANSOURCE=$RPMROOT/SOURCES/freeswan-$FVERSION.tar.gz
KERNELCONFIG=$RPMROOT/SOURCES/kernel-$KVERSION-i586.config
X509PATCH=$RPMROOT/SOURCES/x509-$XVERSION-freeswan-$FVERSION.tar.gz

# Check for freeswan patch already present
if [ -s $FREESWANPATCH ]; then
  echo "Patch already created: $FREESWANPATCH"
  exit 1
fi

# Then make sure the rest is
for i in $KERNELCONFIG $FREESWANSOURCE $KERNELSOURCE $X509PATCH; do
  if [ ! -s $i ]; then
    echo "Could not find $i"
    exit 1
  fi
done


# Unpack sources in /usr/src
cd /usr/src
  tar jxf $KERNELSOURCE
  tar zxf $FREESWANSOURCE
cd freeswan-$FVERSION
  tar zxf $X509PATCH
  # Apply x509 patch to freeswan
  patch -p1 <x509-$XVERSION-freeswan-$FVERSION/freeswan.diff
# Move kernel source to /usr/src/linux
cd /usr/src
  rm -rf linux || :
  mv linux-$KVERSION linux
# Copy kernel .config to /usr/src/linux
cp $KERNELCONFIG linux/.config
make -C /usr/src/linux oldconfig dep
make -C /usr/src/freeswan-$FVERSION precheck verset kpatch ocf
make -C /usr/src/freeswan-$FVERSION kernelpatch2.4 | bzip2 -9 > $RPMROOT/SOURCES/kernel-$KVERSION-freeswan-$FVERSION-x509-$XVERSION.patch.bz2

# Clean up
rm -rf /usr/src/linux
rm -rf /usr/src/freeswan-$FVERSION

# Finished
echo "Success: $RPMROOT/SOURCES/kernel-$KVERSION-freeswan-$FVERSION-x509-$XVERSION.patch.bz2"

